@@ -13,20 +13,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from http import client as http_client
 import logging
+import re
 import time
 from urllib import parse as urlparse
 
 import requests
 
 from sushy import exceptions
-from sushy.resources.task_monitor import TaskMonitor
+from sushy.taskmonitor import TaskMonitor
 from sushy import utils
 
 LOG = logging.getLogger(__name__)
 
 
-_SERVER_SIDE_RETRIES = 5
+_SERVER_SIDE_RETRIES = 10
 _SERVER_SIDE_RETRY_DELAY = 3
 
 
@@ -42,11 +44,23 @@ class Connector(object):
         self._response_callback = response_callback
         self._auth = None
 
+        # NOTE(TheJulia): In order to help prevent recursive post operations
+        # by allowing us to understand that we should stop authentication.
+        self._sessions_uri = None
         # NOTE(etingof): field studies reveal that some BMCs choke at
         # long-running persistent HTTP connections (or TCP connections).
         # By default, we ask HTTP server to shut down HTTP connection we've
         # just used.
         self._session.headers['Connection'] = 'close'
+        # NOTE(TheJulia): Depending on the BMC, offering compression as an
+        # acceptable response changes the ETag behavior to offering an
+        # automatic "weak" ETag response, which is appropriate because the
+        # body content *may* not be a byte for byte match given compression.
+        # Overall, the value of compression is less than the value of concise
+        # interaction with the BMC. Setting to identity basically means
+        # "without modification or compression". By default, python-requests
+        # indicates responses can be compressed.
+        self._session.headers['Accept-Encoding'] = 'identity'
 
         if username or password:
             LOG.warning('Passing username and password to Connector is '
@@ -72,6 +86,15 @@ class Connector(object):
         """Close this connector and the associated HTTP session."""
         self._session.close()
 
+    def check_retry_on_exception(self, exception_msg):
+        """Checks whether retry on exception is required."""
+        if ('SYS518' in str(exception_msg)):
+            LOG.debug('iDRAC is not yet ready after previous operation. '
+                      'Error: %(err)s', {'err': str(exception_msg)})
+            return True
+        else:
+            return False
+
     def _op(self, method, path='', data=None, headers=None, blocking=False,
             timeout=60, server_side_retries=_SERVER_SIDE_RETRIES,
             **extra_session_req_kwargs):
@@ -113,7 +136,14 @@ class Connector(object):
             response = self._session.request(method, url, json=data,
                                              headers=headers,
                                              **extra_session_req_kwargs)
-        except requests.ConnectionError as e:
+        except requests.exceptions.RequestException as e:
+            # Capture any general exception by looking for the parent
+            # class of exceptions in the requests library.
+            # Specifically this will cover cases such as transport
+            # failures, connection timeouts, and encoding errors.
+            # Raising this as sushy ConnectionError allows users to
+            # understand something bad has happened, and to
+            # allow them to respond accordingly.
             raise exceptions.ConnectionError(url=url, error=e)
 
         if self._response_callback:
@@ -126,9 +156,32 @@ class Connector(object):
         try:
             exceptions.raise_for_response(method, url, response)
         except exceptions.AccessError as e:
-            if self._auth is not None and self._auth.can_refresh_session():
+            if (method == 'POST'
+                    and self._sessions_uri is not None
+                    and self._sessions_uri in url):
+                LOG.error('Authentication to the session service failed. '
+                          'Please check credentials and try again.')
+                raise
+            if self._auth is not None:
+                # self._session.auth value is only set when basic auth is used
+                if self._session.auth is not None:
+                    LOG.warning('We have encountered an AccessError when '
+                                'using \'basic\' authentication. %(err)s',
+                                {'err': str(e)})
+                    # NOTE(TheJulia): There is no way to recover Basic auth,
+                    # as we need the client to be re-launched with new
+                    # credentials.
+                    raise
                 try:
-                    self._auth.refresh_session()
+                    if self._auth.can_refresh_session():
+                        self._auth.refresh_session()
+                    else:
+                        LOG.warning('Session authentication appears to have '
+                                    'been lost at some point in time. '
+                                    'Connectivity may have been lost during '
+                                    'a prior session refresh. Attempting to '
+                                    're-authenticate.')
+                        self._auth.authenticate()
                 except exceptions.AccessError as refresh_exc:
                     LOG.error("A failure occured while attempting to refresh "
                               "the session. Error: %s", refresh_exc.message)
@@ -155,9 +208,9 @@ class Connector(object):
                               "%s", e.message)
                 raise
         except exceptions.ServerSideError as e:
-            if method.lower() != 'get' or server_side_retries <= 0:
-                raise
-            else:
+            if ((method.lower() == 'get'
+                or self.check_retry_on_exception(e.message))
+                    and server_side_retries > 0):
                 LOG.warning('Got server side error %s in response to a '
                             'GET request, retrying after %d seconds',
                             e, _SERVER_SIDE_RETRY_DELAY)
@@ -166,28 +219,18 @@ class Connector(object):
                                 blocking=blocking, timeout=timeout,
                                 server_side_retries=server_side_retries - 1,
                                 **extra_session_req_kwargs)
+            else:
+                raise
 
         if blocking and response.status_code == 202:
-            if not response.headers.get('location'):
+            if not response.headers.get('Location'):
                 m = ('HTTP response for %(method)s request to %(url)s '
                      'returned status 202, but no Location header'
                      % {'method': method, 'url': url})
                 raise exceptions.ConnectionError(url=url, error=m)
-            timeout_at = time.time() + timeout
-            mon = (TaskMonitor(self, response.headers.get('location'))
-                   .set_retry_after(response.headers.get('retry-after')))
-            while mon.in_progress:
-                LOG.debug('Blocking for in-progress %(method)s call to '
-                          '%(url)s; sleeping for %(sleep)s seconds',
-                          {'method': method, 'url': url,
-                           'sleep': mon.sleep_for})
-                time.sleep(mon.sleep_for)
-                if time.time() >= timeout_at and mon.in_progress:
-                    m = ('Timeout waiting for blocking %(method)s '
-                         'request to %(url)s (timeout = %(timeout)s)'
-                         % {'method': method, 'url': url,
-                            'timeout': timeout})
-                    raise exceptions.ConnectionError(url=url, error=m)
+
+            mon = TaskMonitor.from_response(self, response, path)
+            mon.wait(timeout)
             response = mon.response
             exceptions.raise_for_response(method, url, response)
 
@@ -238,13 +281,14 @@ class Connector(object):
                         blocking=blocking, timeout=timeout,
                         **extra_session_req_kwargs)
 
-    def patch(self, path='', data=None, headers=None, blocking=False,
-              timeout=60, **extra_session_req_kwargs):
-        """HTTP PATCH method.
+    def _etag_handler(self, path='', data=None, headers=None, etag=None,
+                      blocking=False, timeout=60, **extra_session_req_kwargs):
+        """eTag handler containing workarounds for PATCH requests with eTags.
 
         :param path: Optional sub-URI path to the resource.
         :param data: Optional JSON data.
         :param headers: Optional dictionary of headers.
+        :param etag: eTag string.
         :param blocking: Whether to block for asynchronous operations.
         :param timeout: Max time in seconds to wait for blocking async call.
         :param extra_session_req_kwargs: Optional keyword argument to pass
@@ -254,9 +298,89 @@ class Connector(object):
         :raises: ConnectionError
         :raises: HTTPError
         """
-        return self._op('PATCH', path, data=data, headers=headers,
-                        blocking=blocking, timeout=timeout,
-                        **extra_session_req_kwargs)
+        if etag is not None:
+            if not headers:
+                headers = {}
+            headers['If-Match'] = etag
+        try:
+            response = self._op('PATCH', path, data=data,
+                                headers=headers,
+                                blocking=blocking, timeout=timeout,
+                                **extra_session_req_kwargs)
+        except exceptions.HTTPError as resp:
+            LOG.warning("Initial request with eTag failed: %s",
+                        resp)
+            if resp.status_code == http_client.PRECONDITION_FAILED:
+                # NOTE(janders) if there was no eTag provided AND the response
+                # is HTTP 412 Precondition Failed, raise the exception
+                if not etag:
+                    raise
+                # checking for weak eTag
+                pattern = re.compile(r'^(W\/)("\w*")$')
+                match = pattern.match(etag)
+                if match:
+                    LOG.info("Weak eTag provided with original request to "
+                             "%s. Attempting to conversion to strong eTag "
+                             "and re-trying.", path)
+                    # trying weak eTag converted to strong
+                    headers['If-Match'] = match.group(2)
+                    try:
+                        response = self._op('PATCH', path, data=data,
+                                            headers=headers,
+                                            blocking=blocking,
+                                            timeout=timeout,
+                                            **extra_session_req_kwargs)
+                    except exceptions.HTTPError as resp:
+                        if (resp.status_code == http_client.
+                           PRECONDITION_FAILED):
+                            LOG.warning("Request to %s with weak eTag "
+                                        "converted to strong eTag also "
+                                        "failed. Making the final attempt "
+                                        "with no eTag specified.", path)
+                        response = None
+                    if response:
+                        return response
+                else:
+                    # eTag is strong, if it failed the only other thing
+                    # to try is removing it entirely
+                    # info
+                    LOG.warning("Strong eTag provided - retrying request to "
+                                "%s with eTag removed.", path)
+                del headers['If-Match']
+                try:
+                    response = self._op('PATCH', path, data=data,
+                                        headers=headers,
+                                        blocking=blocking, timeout=timeout,
+                                        **extra_session_req_kwargs)
+                except exceptions.HTTPError as resp:
+                    LOG.error("Final re-try with eTag removed has failed, "
+                              "raising exception %s", resp)
+                    raise
+            else:
+                raise
+        return response
+
+    def patch(self, path='', data=None, headers=None, etag=None,
+              blocking=False, timeout=60, **extra_session_req_kwargs):
+        """HTTP PATCH method.
+
+        :param path: Optional sub-URI path to the resource.
+        :param data: Optional JSON data.
+        :param headers: Optional dictionary of headers.
+        :param etag: Optional eTag string.
+        :param blocking: Whether to block for asynchronous operations.
+        :param timeout: Max time in seconds to wait for blocking async call.
+        :param extra_session_req_kwargs: Optional keyword argument to pass
+         requests library arguments which would pass on to requests session
+         object.
+        :returns: The response object from the requests library.
+        :raises: ConnectionError
+        :raises: HTTPError
+        """
+        return self._etag_handler(path, data,
+                                  headers, etag,
+                                  blocking, timeout,
+                                  **extra_session_req_kwargs)
 
     def put(self, path='', data=None, headers=None, blocking=False,
             timeout=60, **extra_session_req_kwargs):
