@@ -144,7 +144,7 @@ class System(base.ResourceBase):
     _actions = ActionsField('Actions', required=True)
 
     def __init__(self, connector, identity, redfish_version=None,
-                 registries=None):
+                 registries=None, root=None):
         """A class representing a ComputerSystem
 
         :param connector: A Connector instance
@@ -153,9 +153,13 @@ class System(base.ResourceBase):
             the object according to schema of the given version.
         :param registries: Dict of registries to be used in any resource
             that needs registries to parse messages.
+        :param root: Sushy root object. Empty for Sushy root itself.
         """
         super(System, self).__init__(
-            connector, identity, redfish_version, registries)
+            connector, identity,
+            redfish_version=redfish_version,
+            registries=registries,
+            root=root)
 
     def _get_reset_action_element(self):
         reset_action = self._actions.reset
@@ -232,6 +236,13 @@ class System(base.ResourceBase):
             invalid.
         """
         data = collections.defaultdict(dict)
+        settings_data = collections.defaultdict(dict)
+
+        if self._settings and self._settings.resource_uri:
+            settings_resp = self._conn.get(self._settings.resource_uri)
+            settings_boot_section = settings_resp.json().get('Boot', {})
+        else:
+            settings_resp = None
 
         if target is not None:
             valid_targets = self.get_allowed_system_boot_source_values()
@@ -262,7 +273,12 @@ class System(base.ResourceBase):
                           sys_cons.BOOT_SOURCE_TARGET_CD,
                           fishy_target)
 
-            data['Boot']['BootSourceOverrideTarget'] = fishy_target
+            if (settings_resp and "BootSourceOverrideTarget" in
+                    settings_boot_section):
+                settings_data['Boot']['BootSourceOverrideTarget'] = \
+                    fishy_target
+            else:
+                data['Boot']['BootSourceOverrideTarget'] = fishy_target
 
         if enabled is not None:
             if enabled not in sys_maps.BOOT_SOURCE_ENABLED_MAP_REV:
@@ -272,7 +288,11 @@ class System(base.ResourceBase):
 
             fishy_freq = sys_maps.BOOT_SOURCE_ENABLED_MAP_REV[enabled]
 
-            data['Boot']['BootSourceOverrideEnabled'] = fishy_freq
+            if (settings_resp and "BootSourceOverrideEnabled" in
+                    settings_boot_section):
+                settings_data['Boot']['BootSourceOverrideEnabled'] = fishy_freq
+            else:
+                data['Boot']['BootSourceOverrideEnabled'] = fishy_freq
 
         if mode is not None:
             if mode not in sys_maps.BOOT_SOURCE_MODE_MAP_REV:
@@ -282,22 +302,23 @@ class System(base.ResourceBase):
 
             fishy_mode = sys_maps.BOOT_SOURCE_MODE_MAP_REV[mode]
 
-            data['Boot']['BootSourceOverrideMode'] = fishy_mode
-
-        etag = self._get_etag()
-        path = self.path
-
-        # NOTE(janders): checking if @Redfish.Settings are available; if so
-        # using the URI from the SettingsObject. Also ensuring the ETag URI
-        # matches in this case (the one above would not be valid for new URI).
-        if self._settings and self._settings.resource_uri:
-            path = self._settings.resource_uri
-            resp = self._conn.get(path)
-            etag = resp.headers.get('ETag')
+            if (settings_resp and "BootSourceOverrideMode" in
+                    settings_boot_section):
+                settings_data['Boot']['BootSourceOverrideMode'] = fishy_mode
+            else:
+                data['Boot']['BootSourceOverrideMode'] = fishy_mode
 
         # TODO(lucasagomes): Check the return code and response body ?
         #                    Probably we should call refresh() as well.
-        self._conn.patch(path, data=data, etag=etag)
+        if settings_data.get('Boot'):
+            etag = settings_resp.headers.get('ETag')
+            path = self._settings.resource_uri
+            self._conn.patch(path, data=settings_data,
+                             etag=etag)
+        if data.get('Boot'):
+            etag = self._get_etag()
+            path = self.path
+            self._conn.patch(path, data=data, etag=etag)
 
     # TODO(etingof): we should remove this method, eventually
     def set_system_boot_source(
@@ -359,7 +380,7 @@ class System(base.ResourceBase):
         return processor.ProcessorCollection(
             self._conn, self._get_processor_collection_path(),
             redfish_version=self.redfish_version,
-            registries=self.registries)
+            registries=self.registries, root=self.root)
 
     @property
     @utils.cache_it
@@ -374,7 +395,7 @@ class System(base.ResourceBase):
             self._conn,
             utils.get_sub_resource_path_by(self, "EthernetInterfaces"),
             redfish_version=self.redfish_version,
-            registries=self.registries)
+            registries=self.registries, root=self.root)
 
     @property
     @utils.cache_it
@@ -389,7 +410,7 @@ class System(base.ResourceBase):
             self._conn,
             utils.get_sub_resource_path_by(self, 'Bios'),
             redfish_version=self.redfish_version,
-            registries=self.registries)
+            registries=self.registries, root=self.root)
 
     @property
     @utils.cache_it
@@ -411,7 +432,7 @@ class System(base.ResourceBase):
         return sys_simple_storage.SimpleStorageCollection(
             self._conn, utils.get_sub_resource_path_by(self, "SimpleStorage"),
             redfish_version=self.redfish_version,
-            registries=self.registries)
+            registries=self.registries, root=self.root)
 
     @property
     @utils.cache_it
@@ -434,7 +455,7 @@ class System(base.ResourceBase):
         return sys_storage.StorageCollection(
             self._conn, utils.get_sub_resource_path_by(self, "Storage"),
             redfish_version=self.redfish_version,
-            registries=self.registries)
+            registries=self.registries, root=self.root)
 
     @property
     @utils.cache_it
@@ -452,7 +473,7 @@ class System(base.ResourceBase):
 
         return [manager.Manager(self._conn, path,
                                 redfish_version=self.redfish_version,
-                                registries=self.registries)
+                                registries=self.registries, root=self.root)
                 for path in paths]
 
     @property
@@ -471,7 +492,7 @@ class System(base.ResourceBase):
 
         return [chassis.Chassis(self._conn, path,
                                 redfish_version=self.redfish_version,
-                                registries=self.registries)
+                                registries=self.registries, root=self.root)
                 for path in paths]
 
     @property
@@ -487,7 +508,7 @@ class System(base.ResourceBase):
             self._conn,
             utils.get_sub_resource_path_by(self, 'SecureBoot'),
             redfish_version=self.redfish_version,
-            registries=self.registries)
+            registries=self.registries, root=self.root)
 
 
 class SystemCollection(base.ResourceCollectionBase):
@@ -496,7 +517,8 @@ class SystemCollection(base.ResourceCollectionBase):
     def _resource_type(self):
         return System
 
-    def __init__(self, connector, path, redfish_version=None, registries=None):
+    def __init__(self, connector, path, redfish_version=None, registries=None,
+                 root=None):
         """A class representing a ComputerSystemCollection
 
         :param connector: A Connector instance
@@ -505,6 +527,8 @@ class SystemCollection(base.ResourceCollectionBase):
             the object according to schema of the given version.
         :param registries: Dict of Redfish Message Registry objects to be
             used in any resource that needs registries to parse messages
+        :param root: Sushy root object. Empty for Sushy root itself.
         """
         super(SystemCollection, self).__init__(
-            connector, path, redfish_version, registries)
+            connector, path, redfish_version=redfish_version,
+            registries=registries, root=root)
