@@ -19,6 +19,7 @@ from unittest import mock
 
 from sushy.resources import base as sushy_base
 from sushy.resources import constants as res_cons
+from sushy.resources.registry import attribute_registry
 from sushy.resources.registry import message_registry
 from sushy.tests.unit import base
 
@@ -74,6 +75,39 @@ class MessageRegistryTestCase(base.TestCase):
         self.assertEqual(
             'Try Later', self.registry.messages['MissingThings'].resolution)
 
+    def test__parse_attributes_return(self):
+        attributes = self.registry._parse_attributes(self.json_doc)
+
+        self.assertEqual({'Failed':
+                          {'description': 'Nothing is OK',
+                           'message': 'The property %1 broke everything.',
+                           'number_of_args': 1,
+                           'param_types': ['string'],
+                           'resolution': 'Panic',
+                           'severity': 'critical'},
+                          'MissingThings':
+                          {'description': '',
+                           'message':
+                           "Property's %1 value cannot be less than %2.",
+                           'number_of_args': 2,
+                           'param_types': ['string', 'number'],
+                           'resolution': 'Try Later',
+                           'severity': 'warning'},
+                          'Success':
+                          {'description': 'Everything OK',
+                           'message': 'Everything done successfully.',
+                           'number_of_args': 0, 'param_types': None,
+                           'resolution': 'None', 'severity': 'ok'},
+                          'TooBig':
+                          {'description': 'Value too big',
+                           'message':
+                           "Property's %1 value cannot be greater than %2.",
+                           'number_of_args': 2,
+                           'param_types': ['string', 'number'],
+                           'resolution': 'Try again',
+                           'severity': 'warning'}},
+                         attributes.get('messages'))
+
     def test__parse_attributes_missing_msg_desc(self):
         self.json_doc['Messages']['Success'].pop('Description')
         self.registry._parse_attributes(self.json_doc)
@@ -84,7 +118,7 @@ class MessageRegistryTestCase(base.TestCase):
         self.registry._parse_attributes(self.json_doc)
         self.assertEqual('warning', self.registry.messages['Success'].severity)
 
-    def test__parse_attribtues_unknown_param_type(self):
+    def test__parse_attributes_unknown_param_type(self):
         self.registry.json['Messages']['Failed']['ParamTypes'] = \
             ['unknown_type']
         self.assertRaisesRegex(KeyError,
@@ -258,3 +292,28 @@ class MessageRegistryTestCase(base.TestCase):
         self.assertEqual(res_cons.SEVERITY_WARNING, parsed_msg.severity)
         self.assertEqual('Property\'s arg1 value cannot be greater than '
                          'unknown.', parsed_msg.message)
+
+    def test_parse_message_multiple_registries(self):
+        conn = mock.Mock()
+        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
+            conn.get.return_value.json.return_value = json.load(f)
+        msg_registry = message_registry.MessageRegistry(
+            conn, '/redfish/v1/Registries/Test',
+            redfish_version='1.0.2')
+        attr_registry = attribute_registry.AttributeRegistry(
+            self.conn, '/redfish/v1/Test/Bios/BiosRegistry',
+            redfish_version='1.0.2')
+        registries = {'Test.1.0.2': attr_registry,
+                      'Test.1.0.0': msg_registry}
+        message_field = sushy_base.MessageListField('Foo')
+        message_field.message_id = 'Test.1.0.0.TooBig'
+        message_field.message_args = ['arg1', 10]
+        message_field.severity = None
+        message_field.resolution = None
+
+        parsed_msg = message_registry.parse_message(registries, message_field)
+
+        self.assertEqual('Try again', parsed_msg.resolution)
+        self.assertEqual(res_cons.SEVERITY_WARNING, parsed_msg.severity)
+        self.assertEqual('Property\'s arg1 value cannot be greater than 10.',
+                         parsed_msg.message)
