@@ -80,7 +80,9 @@ class SimpleStorageCollectionTestCase(base.TestCase):
         SimpleStorage_mock.assert_called_once_with(
             self.simpl_stor_col._conn,
             '/redfish/v1/Systems/437XR1138R2/SimpleStorage/1',
-            self.simpl_stor_col.redfish_version, None)
+            redfish_version=self.simpl_stor_col.redfish_version,
+            registries=None,
+            root=self.simpl_stor_col.root)
 
     @mock.patch.object(simple_storage, 'SimpleStorage', autospec=True)
     def test_get_members(self, SimpleStorage_mock):
@@ -88,7 +90,8 @@ class SimpleStorageCollectionTestCase(base.TestCase):
         SimpleStorage_mock.assert_called_once_with(
             self.simpl_stor_col._conn,
             '/redfish/v1/Systems/437XR1138R2/SimpleStorage/1',
-            self.simpl_stor_col.redfish_version, None)
+            redfish_version=self.simpl_stor_col.redfish_version,
+            registries=None, root=self.simpl_stor_col.root)
         self.assertIsInstance(members, list)
         self.assertEqual(1, len(members))
 
