@@ -313,9 +313,7 @@ class SystemTestCase(base.TestCase):
 
     def test_set_system_boot_options_nothing_specified(self):
         self.sys_inst.set_system_boot_options()
-        self.sys_inst._conn.patch.assert_called_once_with(
-            '/redfish/v1/Systems/437XR1138R2', data={},
-            etag=None)
+        self.sys_inst._conn.patch.assert_not_called()
 
     def test_set_system_boot_options_invalid_target(self):
         self.assertRaises(exceptions.InvalidParameterValueError,
@@ -362,13 +360,21 @@ class SystemTestCase(base.TestCase):
             etag=None)
 
     def test_set_system_boot_options_settings_resource_nokia(self):
-        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
         with open('sushy/tests/unit/json_samples/settings-nokia.json') as f:
             settings_obj = json.load(f)
 
         self.json_doc.update(settings_obj)
         self.sys_inst._parse_attributes(self.json_doc)
 
+        with open('sushy/tests/unit/json_samples/'
+                  'settings-body-nokia.json') as f:
+            settings_body = json.load(f)
+
+        get_settings = mock.MagicMock(headers={'ETag': '"3d7b838291941d"'})
+        get_settings.json.return_value = settings_body
+        get_system = mock.MagicMock(headers={'ETag': '"222"'})
+        self.conn.get.side_effect = [get_settings, get_system]
+
         self.sys_inst.set_system_boot_options(
             target=sushy.BOOT_SOURCE_TARGET_CD,
             enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
@@ -380,13 +386,23 @@ class SystemTestCase(base.TestCase):
             etag='"3d7b838291941d"')
 
     def test_set_system_boot_options_settings_resource(self):
-        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
         with open('sushy/tests/unit/json_samples/settings.json') as f:
             settings_obj = json.load(f)
 
         self.json_doc.update(settings_obj)
         self.sys_inst._parse_attributes(self.json_doc)
 
+        with open('sushy/tests/unit/json_samples/'
+                  'settings-body-nokia.json') as f:
+            settings_body = json.load(f)
+
+        get_settings = mock.MagicMock(headers={'ETag': '"3d7b838291941d"'})
+        get_settings.json.return_value = settings_body
+        get_system = mock.MagicMock(headers={'ETag': '"222"'})
+        self.conn.get.side_effect = [get_settings, get_system]
+
+        self.conn.get.return_value.json.return_value = settings_body
+
         self.sys_inst.set_system_boot_options(
             target=sushy.BOOT_SOURCE_TARGET_CD,
             enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
@@ -397,6 +413,66 @@ class SystemTestCase(base.TestCase):
                            'BootSourceOverrideTarget': 'Cd'}},
             etag='"3d7b838291941d"')
 
+    def test_set_system_boot_options_settings_resource_lenovo(self):
+        self.sys_inst = system.System(
+            self.conn, '/redfish/v1/Systems/1',
+            redfish_version='1.0.2')
+
+        with open('sushy/tests/unit/json_samples/'
+                  'settings-lenovo-se450.json') as f:
+            settings_obj = json.load(f)
+
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        with open('sushy/tests/unit/json_samples/'
+                  'settings-body-lenovo-se450.json') as f:
+            settings_body = json.load(f)
+
+        get_settings = mock.MagicMock(headers={'ETag': '"3d7b838291941d"'})
+        get_settings.json.return_value = settings_body
+        get_system = mock.MagicMock(headers={'ETag': '"222"'})
+        self.conn.get.side_effect = [get_settings, get_system]
+
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BOOT_SOURCE_TARGET_CD,
+            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/1',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Cd'}},
+            etag='"222"')
+
+    def test_set_system_boot_options_settings_resource_bootmode_only(self):
+        self.sys_inst = system.System(
+            self.conn, '/redfish/v1/Systems/1',
+            redfish_version='1.0.2')
+
+        with open('sushy/tests/unit/json_samples/'
+                  'settings-nokia.json') as f:
+            settings_obj = json.load(f)
+
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        with open('sushy/tests/unit/json_samples/'
+                  'settings-body-bootsourceoverridemode-only.json') as f:
+            settings_body = json.load(f)
+
+        get_settings = mock.MagicMock(headers={'ETag': '"3d7b838291941d"'})
+        get_settings.json.return_value = settings_body
+        get_system = mock.MagicMock(headers={'ETag': '"222"'})
+        self.conn.get.side_effect = [get_settings, get_system]
+
+        self.sys_inst.set_system_boot_options(
+            mode=sushy.BOOT_SOURCE_MODE_UEFI)
+
+        self.sys_inst._conn.patch.assert_called_with(
+            '/redfish/v1/Systems/Self/SD',
+            data={'Boot': {'BootSourceOverrideMode': 'UEFI'}},
+            etag='"3d7b838291941d"')
+
     def test_set_system_boot_source(self):
         self.sys_inst.set_system_boot_source(
             sushy.BOOT_SOURCE_TARGET_PXE,
@@ -784,13 +860,15 @@ class SystemCollectionTestCase(base.TestCase):
         self.sys_col.get_member('/redfish/v1/Systems/437XR1138R2')
         mock_system.assert_called_once_with(
             self.sys_col._conn, '/redfish/v1/Systems/437XR1138R2',
-            self.sys_col.redfish_version, None)
+            redfish_version=self.sys_col.redfish_version, registries=None,
+            root=self.sys_col.root)
 
     @mock.patch.object(system, 'System', autospec=True)
     def test_get_members(self, mock_system):
         members = self.sys_col.get_members()
         mock_system.assert_called_once_with(
             self.sys_col._conn, '/redfish/v1/Systems/437XR1138R2',
-            self.sys_col.redfish_version, None)
+            redfish_version=self.sys_col.redfish_version, registries=None,
+            root=self.sys_col.root)
         self.assertIsInstance(members, list)
         self.assertEqual(1, len(members))
